import { run } from "./utils.ts";
import * as path from "https://deno.land/std@0.214.0/path/mod.ts";

class NginxController {
	proxy_target_url;
	enabled_conf_dir;

	constructor(proxy_target_url: string, enabled_conf_dir: string) {
		this.proxy_target_url = proxy_target_url;
		this.enabled_conf_dir = enabled_conf_dir;
	}

	public async add_proxy(url: string, port: number, conf_dir: string) {
		const conf_file_content = `
server {
    listen 80;
    listen [::]:80;
    server_name ${url};
    location / {
        proxy_pass http://${this.proxy_target_url}:${port};
    }
}
`;
		const conf_file_path = path.join(conf_dir, url + ".conf");
		const enabled_conf_file_path = path.join(this.enabled_conf_dir, url + ".conf");
		await Deno.writeTextFile(conf_file_path, conf_file_content);
		await run("ln", "-s", await Deno.realPath(conf_file_path), enabled_conf_file_path);
		await this.reload();
	}

	public async remove_proxy(url: string, conf_dir: string) {
		const conf_file_path = path.join(conf_dir, url + ".conf");
		const enabled_conf_file_path = path.join(this.enabled_conf_dir, url + ".conf");
		await Deno.remove(enabled_conf_file_path);
		await Deno.remove(conf_file_path);
		await this.reload();
	}

	private async reload() {
		await run("systemctl", "restart", "nginx");
	}
}
